# Copyright (c) Facebook, Inc. and its affiliates.
# All rights reserved.
#
# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.
#
#!/bin/bash

SRC=$1
TRAIN_DIR=$2
TRAIN_SET=$3

if [ "$FAIRSEQ" = "" ]; then
  echo "you must set environment variable FAIRSEQ to fairseq github directory!"
  exit
fi

TGT=en
BPESIZE=20000
TRAIN_MINLEN=1  # remove sentences with <1 BPE tokens
TRAIN_MAXLEN=1020  # remove sentences with >1020 BPE tokens

ROOT="$(cd "$(dirname "$0")"; pwd -P)"
SCRIPTS=$ROOT/spm-scripts
DEV_DIR=$ROOT/dev-sets

SPM_TRAIN=$SCRIPTS/spm_train.py
SPM_ENCODE=$SCRIPTS/spm_encode.py

if [ "$SRC" == "de" ] || [ "$SRC" == "cs" ]; then
  DEV_SET="$DEV_DIR/newstest2017.$SRC-en"
  DEVTEST_SET="$DEV_DIR/newstest2018.$SRC-en"
  MAX_EPOCH=20
  SAVE_INTERVAL=1
elif [ "$SRC" == "hu" ]; then
  DEV_SET="$DEV_DIR/news-dev2009"
  DEVTEST_SET="$DEV_DIR/newstest2009-src"
  MAX_EPOCH=40
  SAVE_INTERVAL=1
elif [ "$SRC" == "et" ]; then
  DEV_SET="$DEV_DIR/newsdev2018.$SRC-en"
  DEVTEST_SET="$DEV_DIR/newstest2018.$SRC-en"
  MAX_EPOCH=40
  SAVE_INTERVAL=1
elif [ "$SRC" == "mt" ]; then
  DEV_SET="$DEV_DIR/EUbookshop.top3k-clean"
  DEVTEST_SET="$DEV_DIR/EUbookshop.next3k"
  MAX_EPOCH=100
  SAVE_INTERVAL=10
else
  echo "unknown language: $SRC"
  exit 1
fi

DATA=$TRAIN_DIR/data
DATABIN=$TRAIN_DIR/data-bin
mkdir -p $DATA $DATABIN

cp $DEV_SET.$SRC $DATA/dev.$SRC
cp $DEV_SET.$TGT $DATA/dev.$TGT
cp $DEVTEST_SET.$SRC $DATA/devtest.$SRC
cp $DEVTEST_SET.$TGT $DATA/devtest.$TGT
cp $TRAIN_SET.f $DATA/train.$SRC
cp $TRAIN_SET.e $DATA/train.$TGT

# learn BPE with sentencepiece
python $SPM_TRAIN \
  --input=$DATA/train.$SRC,$DATA/train.$TGT \
  --model_prefix=$DATA/sentencepiece.bpe \
  --vocab_size=$BPESIZE \
  --character_coverage=1.0 \
  --model_type=bpe

# encode train/dv/test
python $SPM_ENCODE \
  --model $DATA/sentencepiece.bpe.model \
  --output_format=piece \
  --inputs $DATA/train.$SRC $DATA/train.$TGT \
  --outputs $DATA/train.bpe.$SRC $DATA/train.bpe.$TGT \
  --min-len $TRAIN_MINLEN --max-len $TRAIN_MAXLEN
for SPLIT in "dev" "devtest"; do \
  python $SPM_ENCODE \
    --model $DATA/sentencepiece.bpe.model \
    --output_format=piece \
    --inputs $DATA/$SPLIT.$SRC $DATA/$SPLIT.$TGT \
    --outputs $DATA/$SPLIT.bpe.$SRC $DATA/$SPLIT.bpe.$TGT
done

# binarize data
python $FAIRSEQ/preprocess.py \
  --source-lang $SRC --target-lang $TGT \
  --trainpref $DATA/train.bpe --validpref $DATA/dev.bpe --testpref $DATA/devtest.bpe \
  --destdir $DATABIN \
  --joined-dictionary \
  --workers 4

sed "s/\$1/$SRC/" < $ROOT/train.sh | sed "s/\$2/$TRAIN_DIR/" | sed "s/\$3/$MAX_EPOCH/" | sed "s/\$4/$SAVE_INTERVAL/" > $TRAIN_DIR/train.sh
sed "s/\$1/$SRC/" < $ROOT/translate.sh | sed "s/\$2/$TRAIN_DIR/" > $TRAIN_DIR/translate.sh

