#!/usr/bin/perl -w

use strict;

die("subsample.perl FILE_SCORE FILE_F FILE_E OUT") unless scalar(@ARGV) == 4;
my ($FILE_SCORE,$FILE_F,$FILE_E,$OUT) = @ARGV;

# collect word count per score
my %SCORE;
open(E,$FILE_E);
open(SCORE,$FILE_SCORE);
while(my $e = <E>) {
  chop($e);
  my $score = <SCORE>; chop($score);
  my $e_length = scalar split(/ /,$e);
  $SCORE{$score} += $e_length;
}
close(E);
close(SCORE);

# compute thresholds
my %THRESHOLD;
my $count = 0;
my $lowest_score = 0;
foreach my $score (sort {$b <=> $a} (keys %SCORE)) {
  $count += $SCORE{$score};
  $THRESHOLD{$score} = $count;
  $lowest_score = $score;
}
print $count."\n";

# find threshold cutoff values for specified sizes
my @SIZE;
@SIZE = (1e7,2e7,5e7,7e7,1e8,1.5e8,2e8) if $FILE_F =~ /en-de/;
@SIZE = (1e6,1.5e6,2e6,3e6,5e6,7e6,1e7) if $FILE_F =~ /en-mt/;
@SIZE = (5e6,7e6,1e7,1.5e7,2e7,3e7,5e7,7e7) if $FILE_F =~ /en-et/;
@SIZE = (5e6,7e6,1e7,1.5e7,2e7,3e7,5e7) if $FILE_F =~ /en-hu/;
@SIZE = (1e7,1.5e7,2e7,3e7,5e7,7e7,1e8) if $FILE_F =~ /en-cs/;
my %THRESHOLD_CUTOFF;
my %INCLUDED_AT_THRESHOLD;
my $size = 0;
foreach my $score (sort {$b <=> $a} (keys %THRESHOLD)) {
  while ($THRESHOLD{$score} > $SIZE[$size]) { # the first time it goes over threshold...
    print $score."\n";
    $INCLUDED_AT_THRESHOLD{$SIZE[$size]} = $SCORE{$score} - ($THRESHOLD{$score} - $SIZE[$size]);
    $THRESHOLD_CUTOFF{$SIZE[$size++]} = $score;
    last if $size == scalar(@SIZE);
  }
  last if $size == scalar(@SIZE);
}
# is that really needed?
foreach my $size (@SIZE) {
  if (!defined($THRESHOLD_CUTOFF{$size})) {
    $THRESHOLD_CUTOFF{$size} = $lowest_score;
    $INCLUDED_AT_THRESHOLD{$size} = $SCORE{$lowest_score} - ($THRESHOLD{$lowest_score} - $size);
  }
}

# open files to store subsampled sets
my (%OUT_E,%OUT_F);
foreach my $size (@SIZE) {
  open $OUT_E{$size},"> $OUT.$size.e";
  open $OUT_F{$size},"> $OUT.$size.f";
}

# write out sentence pairs scoring over threshold
my %ALREADY_INCLUDED_AT_THRESHOLD;
open(E,$FILE_E);
open(F,$FILE_F);
open(SCORE,$FILE_SCORE);
while(my $e = <E>) {
  chop($e);
  my $f = <F>; chop($f);
  my $score = <SCORE>; chop($score);
  my $e_length = scalar split(/ /,$e);
  foreach my $size (@SIZE) {
    my $cutoff = $THRESHOLD_CUTOFF{$size};
    next if $score < $cutoff;
    if ($score == $cutoff) {
      next if defined($ALREADY_INCLUDED_AT_THRESHOLD{$size}) &&
        $ALREADY_INCLUDED_AT_THRESHOLD{$size} > $INCLUDED_AT_THRESHOLD{$size};
      my $e_length = scalar split(/ /,$e);
      $ALREADY_INCLUDED_AT_THRESHOLD{$size} += $e_length;
    }
    my $fh_e = $OUT_E{$size};
    my $fh_f = $OUT_F{$size};
    print $fh_e "$e\n";
    print $fh_f "$f\n";
  }
}
close(SCORE);
close(F);
close(E);

